# SPDX-License-Identifier: Apache-2.0
# Copyright (C) 2026, The Exgentic organization and its contributors.

"""Compatibility wrapper for litellm cache utilities."""

from .cache import *  # noqa: F403
