# SPDX-License-Identifier: Apache-2.0
# Copyright (C) 2026, The Exgentic organization and its contributors.

"""BFCL benchmark adapter."""

from .bfcl_benchmark import BFCLBenchmark

__all__ = ["BFCLBenchmark"]
